!=~~=~~=~~=~~=~~=~~=~~=~~=~~=~~=~~=~~=~~=~~=~~=~~=~~=~~=~~=~~=~~=~~=~~=~~=~~=~~=~~=~~=~~=~~=~~=~~=~~=~=~
!                                          numgeo
!                Copyright (C) 2018-2023 Jan Machacek, Patrick Staubach
!=~~=~~=~~=~~=~~=~~=~~=~~=~~=~~=~~=~~=~~=~~=~~=~~=~~=~~=~~=~~=~~=~~=~~=~~=~~=~~=~~=~~=~~=~~=~~=~~=~~=~=~
!
! SUBROUTINE: user_contact_properties
!
!> author: Jan Machacek, jan.machacek@outlook.de
!> date: 06.12.2022
!
!> Interface to user subroutine to define amplitude value
!
!>### History
!> 06.12.2022, J. Machacek - Initial version
!> 25.08.2023, J. Machacek - Extended interface by nchar
!
!=~~=~~=~~=~~=~~=~~=~~=~~=~~=~~=~~=~~=~~=~~=~~=~~=~~=~~=~~=~~=~~=~~=~~=~~=~~=~~=~~=~~=~~=~~=~~=~~=~~=~=~
subroutine user_material(material_name, nchar, ielem ,igp ,istep ,iinc ,ntens ,nprops ,nstatev , &
                         strain ,dstrain ,coords ,time ,dtime ,props ,statev ,stress ,dds_dde) bind(c,name='user_material')
  use, intrinsic :: iso_c_binding
  implicit none
  character(c_char), intent(in) :: material_name (*)
  integer(c_int), intent(in) :: nchar
  integer(c_int), intent(in) :: ielem
  integer(c_int), intent(in) :: igp
  integer(c_int), intent(in) :: istep
  integer(c_int), intent(in) :: iinc
  integer(c_int), intent(in) :: ntens
  integer(c_int), intent(in) :: nprops
  integer(c_int), intent(in) :: nstatev
  real(c_double), dimension(ntens), intent(in) :: strain
  real(c_double), dimension(ntens), intent(in) :: dstrain
  real(c_double), dimension (3), intent(in) :: coords
  real(c_double), intent(in) :: time
  real(c_double), intent(in) :: dtime
  real(c_double), dimension(nprops), intent(in) :: props
  real(c_double), dimension(nstatev), intent(inout) :: statev 
  real(c_double), dimension(ntens), intent(inout) :: stress
  real(c_double), dimension(ntens ,ntens), intent(inout) :: dds_dde
  character(len=nchar) :: material_str
  
  real(c_double) :: E0, nu, n, pref
  real(c_double) :: E, fp, p
  real(c_double) :: k1, k2, k3
  
  ! required to make the material name usable in select case of if-clauses
  material_str = transfer(material_name(1:nchar), material_str)

  dds_dde = 0.0d0
  E0   = props(1)
  nu   = props(2)
  n    = props(3)
  pref = props(4)
  
  p = -sum(stress(1:3))/3.0d0
  
  fp = (max(pref,p)/pref)**n
  
  E = E0 * fp
  
  k1 = nu * E / (1.0d0+nu) / (1.0d0-2.0d0*nu)
  k2 = E / 2.0d0 / (1.0d0+nu)
  k3 = k1 + 2.0d0 * k2
  
  dds_dde(1:3,1:3) = k1
  dds_dde(1,1) = k3
  dds_dde(2,2) = k3
  dds_dde(3,3) = k3
  dds_dde(4,4) = k2
  
  if (ntens == 6) then
    dds_dde(5,5) = k2
    dds_dde(6,6) = k2
  end if

  stress = stress + matmul(dds_dde,dstrain)
  
  statev(1) = fp
  statev(2) = E

end subroutine user_material
